package fi.fa.opplugin;

import com.alibaba.druid.util.StringUtils;
import fi.fa.opplugin.utils.ChangeDataUtils;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.list.BillList;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import java.util.EventObject;
import java.util.List;

//资产报废单中间表下推业务处理单
public class FaDisCardMidOpPlugin extends AbstractListPlugin {

    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners(new String[]{"toolbarap"});
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        String itemKey = evt.getItemKey();
        if ("nckd_push".equals(itemKey)) {
            StringBuilder errMsg = new StringBuilder();
            BillList billList = (BillList)this.getControl("billlistap");
            ListSelectedRowCollection list =  billList.getSelectedRows();
            Object[] primaryKeyValues = list.getPrimaryKeyValues();
            if (primaryKeyValues.length > 0) {
                for (Object pk : primaryKeyValues) {
                    DynamicObject info = BusinessDataServiceHelper.loadSingle(pk, "nckd_e_discard_mid");
                    String nckd_flag = info.getString("nckd_flag");
                    if ("1".equals(nckd_flag)) {
                        errMsg.append("单据号: ").append(info.getString("number")).append("已经下推生成了业务处理单，不允许重复下推!");
                        errMsg.append("\r\n");
                        continue;
                    }
                    String error = ChangeDataUtils.check(info);
                    if (!StringUtils.isEmpty(error)) {
                        errMsg.append(error);
                        errMsg.append("\r\n");
                        continue;
                    }

                    List<String> codeList = ChangeDataUtils.getCardNumber(info);
                    DynamicObject busProcess = BusinessDataServiceHelper.newDynamicObject("nckd_ywcld");
                    busProcess.set("nckd_bizdate", info.getDate("nckd_date_out"));//业务日期
                    busProcess.set("nckd_detailtype", "D");//业务类型 D:报废
                    busProcess.set("nckd_costcompany", ChangeDataUtils.getOrg(info.getString("nckd_org_in")));//资产组织
                    busProcess.set("nckd_textfield1", info.getString("nckd_reason"));//摘要事由
                    busProcess.set("nckd_applierv", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//制单人

                    //制单信息
                    busProcess.set("creator", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//创建人
                    busProcess.set("createtime", info.getDate("nckd_date_out"));//创建时间
                    busProcess.set("billstatus", "A");//状态
                    QFilter filter = new QFilter("number", QCP.in,codeList);
                    filter.and("isbak",  QCP.equals, "0");
                    DynamicObject[] card = BusinessDataServiceHelper.load("fa_card_real","id",new QFilter[] {filter});
                    for (int c = 0; c < card.length; c++) {
                        DynamicObject cardInfo = BusinessDataServiceHelper.loadSingle(card[c].getPkValue(), card[c].getDynamicObjectType().getName());
                        //创建报废明细分录
                        DynamicObjectCollection entryEntity = busProcess.getDynamicObjectCollection("nckd_entryentity6");
                        DynamicObject entry1 = entryEntity.addNew();
                        entry1.set("nckd_basedatafield", cardInfo);//资产编码
                        String nckd_count = info.getString("nckd_count").isEmpty() ? "1" : info.getString("nckd_count");
                        Integer count = Integer.valueOf(nckd_count);
                        entry1.set("nckd_integerfield", count);//报废数量
                    }
                    //执行保存-提交-审核
                    OperationResult saveResult = SaveServiceHelper.saveOperate("nckd_ywcld", new DynamicObject[]{busProcess}, OperateOption.create());
                    if (saveResult.isSuccess()){
                        info.set("nckd_flag", "1");
                        SaveServiceHelper.save(new DynamicObject[]{info});
                        errMsg.append("单据号: ").append(info.getString("number")).append("下推生成业务处理单成功!");
                        errMsg.append("\r\n");
                        OperationResult submitResult = OperationServiceHelper.executeOperate("submit", "nckd_ywcld",saveResult.getSuccessPkIds().toArray(),OperateOption.create());
                        OperationResult auditResult = OperationServiceHelper.executeOperate("audit", "nckd_ywcld",submitResult.getSuccessPkIds().toArray(),OperateOption.create());
                    }else {
                        errMsg.append(saveResult.getMessage());
                        errMsg.append("\r\n");
                    }
                }
            }else {
                this.getView().showMessage("请至少选择一条数据");
            }
            this.getView().showMessage(errMsg.toString());
        }
    }
}
